/*//////////////////////////////////////////////////////////////////////////////
//    ##      #####   ######   ######     ##     ####     #######  ##  ##     //
//   ####    ##   ##  # ## #    ##  ##   ####     ##       ##   #  ##  ##     //
//  ##  ##   #          ##      ##  ##  ##  ##    ##       ## #     ####      //
//  ##  ##    #####     ##      #####   ##  ##    ##       ####      ##       //
//  ######        ##    ##      ## ##   ######    ##   #   ## #     ####      //
//  ##  ##   ##   ##    ##      ##  ##  ##  ##    ##  ##   ##   #  ##  ##     //
//  ##  ##    #####    ####    #### ##  ##  ##   #######  #######  ##  ##     //
//----------------------------------------------------------------------------//
//----------------------------------------------------------------------------//
//      #####   ##   ##    ##     #####    #######  ######    #####           //
//     ##   ##  ##   ##   ####     ## ##    ##   #   ##  ##  ##   ##          //
//     #        ##   ##  ##  ##    ##  ##   ## #     ##  ##  #                //
//      #####   #######  ##  ##    ##  ##   ####     #####    #####           //
//          ##  ##   ##  ######    ##  ##   ## #     ## ##        ##          //
//          ##  ##   ##  ######    ##  ##   ## #     ## ##        ##          //
//     ##   ##  ##   ##  ##  ##    ## ##    ##   #   ##  ##  ##   ##          //
//      #####   ##   ##  ##  ##   #####    #######  #### ##   #####           //
/////(BSL Shaders Edit)//////////////////////////////////////By LexBoosT//////*/

//Settings//

#include "/lib/util/fastMath.glsl"

#include "/settings/globalSettings.glsl"

//Varyings//
varying vec2 texCoord;

//Fragment Shader///////////////////////////////////////////////////////////////////////////////////
#ifdef FSH

//Uniforms//

uniform sampler2D colortex1;

float n = float((KUWAHARA_RADIUS + 1.0) * (KUWAHARA_RADIUS + 1.0));

void kuwastep(inout vec3 m, inout float s, in ivec2 dir){
    vec3 newm = vec3(0.0);
    vec3 news = vec3(0.0);
    vec3 c;

    for(int i =0; i<=KUWAHARA_RADIUS; i++){
        for(int j =0; j<=KUWAHARA_RADIUS; j++){
            c = texelFetch(colortex1, texelCoord + ivec2(i, j) * dir, 0).rgb;
            newm += c;
            news += c * c;
        }
    }
    newm /= n;
    news = abs(news / n - newm * newm);

    float sigma = news.r + news.g + news.b;
    if(sigma < s){
        s = sigma;
        m = newm;
    }
}

vec3 kuwahara(){

    vec3 color = vec3(0.0);
    float s = 1000.0;

    kuwastep(color, s, ivec2(1,1));
    kuwastep(color, s, ivec2(1,-1));
    kuwastep(color, s, ivec2(-1,-1));
    kuwastep(color, s, ivec2(-1,1));

    return color;
}

    //Program//
void main() {

    vec4 color = texelFetch(colortex1, texelCoord, 0);

    #ifdef KUWAHARA_FILTER
        color.rgb = kuwahara();
    #endif

        /*DRAWBUFFERS:1*/
    gl_FragData[0] = vec4(color);
}

#endif

//Vertex Shader/////////////////////////////////////////////////////////////////////////////////////
#ifdef VSH

    //Program//
void main() {
    texCoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

    gl_Position = ftransform();
}

#endif